﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Win32;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace CKMusicPlayer
{
    ////////////////////////////////////////////////////////////////////////////////
    // [Common for CKMusicPlayer]
    ////////////////////////////////////////////////////////////////////////////////
    //【変更履歴】 
    //
    // 
    ////////////////////////////////////////////////////////////////////////////////

    public partial class CC
    {
        //*** 定数
        public const MessageBoxButtons BTN_OK = MessageBoxButtons.OK;
        public const MessageBoxButtons BTN_OKCANCEL = MessageBoxButtons.OKCancel;
        public const MessageBoxIcon ICON_ERR = MessageBoxIcon.Error;
        public const MessageBoxIcon ICON_EXC = MessageBoxIcon.Exclamation;
        public const MessageBoxIcon ICON_INF = MessageBoxIcon.Information;
        public const MessageBoxIcon ICON_QTN = MessageBoxIcon.Question;
        public const DialogResult RSLT_CANCEL = DialogResult.Cancel;
        public const DialogResult RSLT_OK = DialogResult.OK;
        public const string ERR = "ERR";
        public const string EXC = "EXC";
        public const string INF = "INF";
        public const string MSG_TTL = "Music Player";
        public const string QT = "\"";
        //public const string SKEY = 
        //    @"software\VB and VBA Program Settings\Clikington\CKMusicPlayer";

        public static readonly Color COL_CTR = SystemColors.Control;
        public static readonly Color COL_WIN = SystemColors.Window;
        public static string NL = Environment.NewLine;
        
        //*** グローバルメソッド

        /// <summary> [CheckNoInvalidFileNameChars]メソッド
        /// ファイル名として使用不可能な文字が含まれていないことを確認する
        /// </summary>
        /// <param name="vtext"></param>
        /// <returns>false:使用不可能文字を含む、true:含まない</returns>
        public static bool CheckNoInvalidFileNameChars(string vtext)
        {
            char[] invalidch = Path.GetInvalidFileNameChars();
            var lstInv = new List<string>();

            foreach (char c in invalidch)
            {
                if (vtext.Contains(c.ToString()))
                    lstInv.Add(c.ToString());
            }
            if (lstInv.Count > 0)
            {
                string schars = String.Empty;
                for (int i = 0; i < lstInv.Count; i++)
                {
                    schars += lstInv[i] + " ";
                }
                ShowMsg(schars + CC.NL + "を含む文字列は使用できません。", EXC);
                return false;
            }
            else
                return true;
        }

        /// <summary> [CheckNoInvalidFileNameChars]メソッド(オーバーロード)
        /// ファイル名として使用不可能な文字が含まれていないことを確認する
        /// </summary>
        /// <param name="vtb"></param>
        /// <param name="vtext"></param>
        /// <returns>false:使用不可能文字を含む、true:含まない</returns>
        public static bool CheckNoInvalidFileNameChars(TextBox vtb, string vtext)
        {
            char[] invalidch = Path.GetInvalidFileNameChars();
            var lstInv = new List<string>();

            foreach (char c in invalidch)
            {
                if (vtext.Contains(c.ToString()))
                    lstInv.Add(c.ToString());
            }
            if (lstInv.Count > 0)
            {
                string schars = String.Empty;
                for (int i = 0; i < lstInv.Count; i++)
                {
                    schars += lstInv[i] + " ";
                }
                vtb.BackColor = Color.Red;
                ShowMsg(schars + CC.NL + "を含む文字列は使用できません。", EXC);
                vtb.BackColor = COL_WIN;
                return false;
            }
            else
                return true;
        }

        /// <summary> [IsDigit]
        /// 引数文字列の全文字が数字である場合にtrue、否の場合falseを返す
        /// </summary>
        /// <param name="vData"></param>
        /// <returns>true:OK, false:引数文字列に数字以外のものが含まれている</returns>
        public static bool IsDigit(string vnum)
        {
            if (System.Text.RegularExpressions.Regex.IsMatch(vnum, "^[0-9]+$"))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary> [IsNarrow]メソッド </summary>
        /// <param name="vtext"></param>
        /// <returns>true:引数が半角文字列の場合、false:引数に全角文字が含まれる場合</returns>
        public static bool IsNarrow(string vtext)
        {
            return (vtext.Length == NByteCount(vtext));
        }

        /// <summary> [MsgCanceled]
        /// OKCancelのMessageBoxを表示し、結果がCancelの場合にtrueを返す
        /// </summary>
        /// <param name="vMsg"></param>
        /// <returns>true:DialogResult.Cancel, false:DialogResult.OK</returns>
        public static bool MsgCanceled(string vMsg)
        {
            if (MessageBox.Show(vMsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN) == RSLT_CANCEL)
                return true;
            else
                return false;
        }

        /// <summary> [MsgCanceled]：([MsgCanceled]のオーバーロード)
        /// </summary>
        /// <param name="vmsg"></param>
        /// <param name="vDefCancel">trueの場合、Button2をDefaultButtonにする</param>
        /// <returns>true:DialogResult.Cancel, false:DialogResult.OK</returns>
        public static bool MsgCanceled(string vmsg, bool vDefCancel)
        {
            if (vDefCancel)
            {
                if (MessageBox.Show(vmsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN,
                        MessageBoxDefaultButton.Button2) == RSLT_CANCEL)
                    return true;
                else
                    return false;
            }
            else
            {
                if (MessageBox.Show(vmsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN,
                        MessageBoxDefaultButton.Button1) == RSLT_CANCEL)
                    return true;
                else
                    return false;
            }
        }

        /// <summary> [GetAllControls]
        /// コンテナコントロール内の全コントロールを配列で取得する
        /// 　.NET_Tips(2004/10/29)から取得
        /// </summary>
        /// <param name="top">コンテナコントロール</param>
        /// <returns>コントロールの配列</returns>
        public static Control[] GetAllControls(Control vtop)
        {
            List<Control> buf = new List<Control>();
            foreach (Control c in vtop.Controls)
            {
                buf.Add(c);
                buf.AddRange(GetAllControls(c));
            }
            return buf.ToArray();
        }

        /// <summary> [NByteCount]：引数文字列のバイト数を返す </summary>
        /// <param name="vstr"></param>
        /// <returns></returns>
        public static int NByteCount(string vstr)
        {
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
            return sjisEnc.GetByteCount(vstr);
        }

        /// <summary> [CheckDir]：ディレクトリの末尾が"\"でないときは、"\"を付ける
        /// </summary>
        /// <param name="vdir"></param>
        /// <returns></returns>
        public static string CheckDir(string vdir)
        {
            string result = vdir;
            if (!vdir.EndsWith("\\"))
                result += "\\";

            return result;
        }

        /// <summary> [DirAdjusted]：ディレクトリの末尾が"\"でないときは、"\"を付ける
        /// </summary>
        /// <param name="vdir"></param>
        /// <returns></returns>
        public static string DirAdjusted(string vdir)
        {
            string result = vdir;
            if (!result.EndsWith("\\")) result += "\\";

            return result;
        }

        /// <summary> [GetRegistry]メソッド </summary>
        /// <param name="vkey"></param>
        /// <param name="vitem"></param>
        /// <param name="vdef">既定値</param>
        /// <returns></returns>
        public static string GetRegistry(string vkey, string vitem, string vdef)
        {
            string data = vdef;
            try
            {
                RegistryKey skey = Registry.CurrentUser.OpenSubKey(vkey);
                if (skey == null)
                    skey = Registry.CurrentUser.CreateSubKey(vkey);
                try
                {
                    data = (string)skey.GetValue(vitem);
                }
                catch { }
                skey.Close();
            }
            catch { }

            if (data == null) data = vdef;
            return data;
        }

        /// <summary> [StringBetweenComma]メソッド </summary>
        /// <param name="vstr"></param>
        /// <returns></returns>
        public static string StringBetweenComma(string vstr)
        {
            string result = String.Empty;

            if (vstr.Length == 0) return result;

            int pos1 = vstr.IndexOf("\"");
            int pos2 = 0;
            if (pos1 >= 0)
            {
                pos2 = vstr.IndexOf("\"", pos1 + 1);
            }
            if (pos2 > pos1 + 1)
            {
                result = vstr.Substring(pos1 + 1, pos2 - pos1 - 1);
            }

            return result;
        }

        /// <summary> [CtrlKeyDown]：[Enter]キーにより、コントロールのFocusを前後させる
        /// </summary>
        /// <param name="vForm"></param>
        /// <param name="e"></param>
        public static void CtrlKeyDown(Form vForm, KeyEventArgs e)
        {
            bool bMove;
            if (e.KeyCode == Keys.Enter && (!e.Control))
            {
                if (e.Modifiers == Keys.Shift)
                    bMove = false;
                else
                    bMove = true;

                e.SuppressKeyPress = true;
                vForm.SelectNextControl(vForm.ActiveControl, bMove, true, true, true);
            }
        }

        /// <summary> [SetForm]：FormのLocationをセンターより少し上にセットする
        /// </summary>
        /// <param name="form"></param>
        public static void SetForm(Form form)
        {
            form.Left = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Width - form.Width) / 2);
            form.Top = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Height - form.Height) / 3);
        }

        /// <summary> [SetGridProperty]：DataGridViewのプロパティをセットする </summary>
        /// <param name="vgrid"></param>
        public static void SetGridProperty(DataGridView vgrid)
        {
            vgrid.AllowUserToAddRows = false;
            vgrid.AllowUserToDeleteRows = false;
            vgrid.AllowUserToOrderColumns = false;
            vgrid.AllowUserToResizeColumns = false;
            vgrid.AllowUserToResizeRows = false;
            vgrid.ColumnHeadersHeight = 25;
            vgrid.ColumnHeadersHeightSizeMode =
                DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            vgrid.ColumnHeadersDefaultCellStyle.Alignment =
                DataGridViewContentAlignment.MiddleCenter;
            vgrid.ColumnHeadersDefaultCellStyle.Font =
                new Font("ＭＳ ゴシック", 12, FontStyle.Bold);
            vgrid.DefaultCellStyle.Font = new Font("ＭＳ ゴシック", 12, FontStyle.Bold);
            vgrid.MultiSelect = false;
            vgrid.ReadOnly = true;
            vgrid.RowHeadersVisible = false;
            vgrid.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            //*** ColumnHeaderのClickによるSortを無効にする
            for (int i = 0; i < vgrid.Columns.Count; i++)
            {
                vgrid.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            }
        }

        /// <summary> [SetRegistry]メソッド </summary>
        /// <param name="vkey"></param>
        /// <param name="vitem"></param>
        /// <param name="vdata"></param>
        public static void SetRegistry(string vkey, string vitem, string vdata)
        {
            if (vkey.Length * vitem.Length != 0)
            {
                RegistryKey skey = Registry.CurrentUser.CreateSubKey(vkey);
                skey.SetValue(vitem, vdata);
                skey.Close();
            }
        }

        /// <summary> [ShowAlarm]メソッド </summary>
        /// <param name="vctrl"></param>
        /// <param name="vmsg"></param>
        public static void ShowAlarm(Control vctrl, string vmsg)
        {
            vctrl.BackColor = Color.Red;
            CC.ShowMsg(vmsg, CC.EXC);
            vctrl.BackColor = CC.COL_WIN;
        }

        /// <summary> [ShowMsg]：INFORMATION、EXCLAMATIONのMessageBoxを出す
        /// </summary>
        /// <param name="vmsg"></param>
        /// <param name="vmode"></param>
        public static void ShowMsg(string vmsg, string vmode)
        {
            if (vmode == INF)
                MessageBox.Show(vmsg, MSG_TTL, BTN_OK, ICON_INF);
            else
                MessageBox.Show(vmsg, MSG_TTL, BTN_OK, ICON_EXC);
        }
    }

}
