﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Shell32;
using System.IO;

namespace CKMusicPlayer
{
    public partial class F00_Main : Form
    {
        /// <summary> [コンストラクター] </summary>
        public F00_Main()
        {
            InitializeComponent();
        }

        const double INTERVAL = 1;   // 終点設定秒数
        const int PCOUNT = 1;        // 

        private bool _eventGrdSelection = true;
        private bool _trbClicked = false;
        private double _max = 0;
        private double _ratio = 1;
        private int _volume;         // 再生中の曲の音量、[PlayStart]で取得
        private List<double> _lstVRatio = new List<double>();

        /// <summary> [参照]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetAlbum_Click(object sender, EventArgs e)
        {
            const string DEF_FOLDER = @"C:\";
            string iniFolder = Properties.Settings.Default.AlbumFolder;
            if (!Directory.Exists(iniFolder)) iniFolder = DEF_FOLDER;

            if (!CC.IsWin7 || CC.UseFolderDlg)
            {
                folderDlg.Description = "フォルダを選択してください。";
                folderDlg.ShowNewFolderButton = false;
                folderDlg.SelectedPath = iniFolder;

                if (folderDlg.ShowDialog() == CC.RSLT_OK)
                    GetAlbum(folderDlg.SelectedPath);
            }
            else
            {
                using (var fcom = new Fcom_FolderDlg())
                {
                    fcom.SelectedPath = iniFolder;
                    if (fcom.ShowDialog() == CC.RSLT_OK)
                        GetAlbum(fcom.SelectedPath);
                }
            }   
        }

        /// <summary> [≫]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNext_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;

            wmPlayer.Ctlcontrols.stop();
            ToNext();
        }

        /// <summary> [∥]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPause_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            wmPlayer.Ctlcontrols.pause();
            btnStart.Enabled = true;
            btnPause.Enabled = false;
            //label1.Text=grdView.Rows[_sidx].Cells[CC.CLM_TITLE].Value.ToString();
            //label2.Text = String.Empty;
        }

        /// <summary> [≪]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPrevious_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;

            wmPlayer.Ctlcontrols.stop();
            ToPrevious();
        }

        /// <summary> [＞]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            PlayStart();
        }

        /// <summary> [■]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            wmPlayer.Ctlcontrols.stop();
            pgbar.Value = 0;
            btnPause.Enabled = false;
            if (grdView.Rows.Count > 0) btnStart.Enabled = true;
        }

        /// <summary> [cbxCustom_SIChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxCustom_SIChanged(object sender, EventArgs e)
        {
            if (cbxCustom.SelectedIndex < 0) return;

            string filefp = CC.CUST_FOLD + @"\" + cbxCustom.Text + ".txt";
            if (!File.Exists(filefp))      // 通常、この事態は発生しない
            {
                cbxCustom.BackColor = Color.Red;
                CC.ShowMsg("このリストは存在しません。", CC.EXC);
                cbxCustom.BackColor = CC.COL_WIN;
                cbxCustom.SelectedIndex = -1;
                return;
            }

            grdView.Rows.Clear();

            wmPlayer.Ctlcontrols.stop();
            pgbar.Value = 0;
            DisableButtons();
            _ratio = 1;

            _eventGrdSelection = false;
            bool result;
            ReadList(filefp, out result);
            _eventGrdSelection = true;

            if (result)
            {
                grdView.Rows[0].Selected = true;
                if (grdView.Rows.Count > 1)
                {
                    btnNext.Enabled = true;
                    btnPrevious.Enabled = true;
                }
                btnPause.Enabled = false;
                btnStart.Enabled = true;
                trkbar.Value = 0;
            }
        }

        /// <summary> [Form_Closing]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closing(object sender, FormClosingEventArgs e)
        {
            if (wmPlayer.playState != CC.STT_STOP 
                        && wmPlayer.playState != CC.STT_UDFD)
                wmPlayer.Ctlcontrols.stop();
            _lstVRatio = null;
        }

        /// <summary> [Form_Load]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            CC.SetForm(this); ;

            string ver =
                System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
            this.Text += " (Ver " + ver + ")";

            InitializeForm();
        }
        
        /// <summary> [grdView_CellClick]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            timer1.Enabled = false;

            wmPlayer.Ctlcontrols.stop();
            pgbar.Value = 0;

            if (grdView.Rows.Count > 0)    // Ver1.1.0で追加
                PlayStart();
        }

        /// <summary> [grdView_SelectionChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_SelectionChanged(object sender, EventArgs e)
        {
            if (!_eventGrdSelection) return;  // アルバム参照時の[PutData2GrdView]を回避する

            int idx = (grdView.SelectedRows.Count > 0) ? grdView.SelectedRows[0].Index : -1;

            if (idx < 0)
            {
                lblArtist.Text = String.Empty;
                lblCurrent.Text = String.Empty;
            }
            else
            {
                lblArtist.Text = grdView.Rows[idx].Cells[CC.CLM_ART_XP].Value.ToString();
                lblCurrent.Text = grdView.Rows[idx].Cells[CC.CLM_TITLE].Value.ToString();
            }
        }

        /// <summary> [カスタムリスト編集]メニュークリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuCustom_Click(object sender, EventArgs e)
        {
            using (var sform = new F10_CustomList())
            {
                sform.CustomList = (rdoCustom.Checked) ? cbxCustom.Text: String.Empty;
                sform.ShowDialog();
                if (sform.CreatedOrDeleted)
                {
                    FillCbxCustom();
                    rdoCustom.Enabled = (cbxCustom.Items.Count > 0);
                }
            }
        }

        /// <summary> [標準フォルダーダイアログ使用]メニュークリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuStandardFolder_Click(object sender, EventArgs e)
        {
            CC.UseFolderDlg = mnuStandardFolder.Checked;
            Properties.Settings.Default.StandardFolder = mnuStandardFolder.Checked;
            Properties.Settings.Default.Save();
        }

        /// <summary> [基準音量]メニュークリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuVolume_Click(object sender, EventArgs e)
        {
            if (wmPlayer.playState != CC.STT_STOP 
                && wmPlayer.playState != CC.STT_UDFD)
            {
                string msg = "基準音量を設定する際には、演奏を停止してください。";
                CC.ShowMsg(msg, CC.EXC);
                return;
            }

            using (var sform = new F20_StdVolume())
            {
                sform.FLeft = this.Left + 200;
                sform.FTop = this.Top + 40;
                if (sform.ShowDialog() == CC.RSLT_OK)
                {
                    if (grdView.Rows.Count > 0)
                    { 
                        ResetVolume();
                        _ratio = 1;
                    }
                }
            }
        }

        /// <summary> [Rdo_CheckedChanged]イベント
        /// [rdoAlbum],[rdoCustom]に対して適用
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Rdo_CheckedChanged(object sender, EventArgs e)
        {
            var rbtn = (RadioButton)sender;
            if (!rbtn.Checked) return;

            if (wmPlayer.playState != CC.STT_STOP)
                wmPlayer.Ctlcontrols.stop();
            grdView.Rows.Clear();
            //_sidx = -1;
            DisableButtons();
            trkbar.Value = 0;
                        
            if (rbtn == rdoAlbum)
            {
                btnGetAlbum.Enabled = true;
                txtAlbum.BackColor = CC.COL_WIN;
                cbxCustom.SelectedIndex = -1;
                cbxCustom.Enabled = false;
            }
            else
            {
                btnGetAlbum.Enabled = false;
                txtAlbum.BackColor = CC.COL_CTR;
                txtAlbum.Text = String.Empty;
                cbxCustom.Enabled = true;
            }
        }

        /// <summary> [timer1_Tick]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            double duration = wmPlayer.currentMedia.duration;
            double current = wmPlayer.Ctlcontrols.currentPosition;
            if (duration > 0) pgbar.Maximum = (int)duration;
            pgbar.Value = (int)current;
            _max = Math.Max(_max, current);
            lblTime.Text = MinSec(current) + " / " + MinSec(duration);
            Application.DoEvents();
            if (duration <= current + INTERVAL) ToNext();
        }

        /// <summary> [trkbar_MouseDown]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trkbar_MouseDown(object sender, MouseEventArgs e)
        {
            _trbClicked = true;            
        }

        /// <summary> [trkbar_MouseUp]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trkbar_MouseUp(object sender, MouseEventArgs e)
        {
            if (_trbClicked)
            {
                //int vol = (int)((Math.Round(trkbar.Value * 0.2)) * 5);
                //trkbar.Value = vol;
                int vol = trkbar.Value;
                lblVolume.Text = vol.ToString();
                if (wmPlayer.playState == CC.STT_PLAY)
                    wmPlayer.settings.volume = vol;
                _ratio = (double)vol / (double)_volume;
                _trbClicked = false;
            }
        }

        /// <summary> [trkbar_ValueChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trkbar_ValueChanged(object sender, EventArgs e)
        {
            if (!_trbClicked)
                lblVolume.Text = trkbar.Value.ToString();
        }


        /// <summary> [IsExistSong]メソッド
        /// [GetAlbum]から呼出し
        /// </summary>
        /// <param name="vfp">フォルダー名(フルパス)</param>
        /// <param name="vlstSongs"></param>
        /// <returns></returns>
        private bool IsExistSong(string vfp, ref List<string> vlstSongs)
        {
            if (vfp.Length == 0) return false;

            vlstSongs = new List<string>();
            foreach (string sfile in Directory.GetFiles(vfp))
            {
                for (int i = 0; i < CC.EXTS.Length; i++)
                {
                    //string ext = Path.GetExtension(sfile).ToLower();
                    if (Path.GetExtension(sfile).ToLower() == CC.EXTS[i])
                    {
                        vlstSongs.Add(sfile);
                        break;
                    }
                }
            }
            return (vlstSongs.Count > 0);
        }

        /// <summary> [MinSec]メソッド
        /// [timer1_Tick]から呼出し
        /// </summary>
        /// <param name="vtime"></param>
        /// <returns></returns>
        private string MinSec(double vtime)
        {
            int itime = (int)vtime;
            int min = itime / 60;
            int sec = itime % 60;

            return min.ToString().PadLeft(3, '0') + ":" + sec.ToString().PadLeft(2, '0');
        }

        /// <summary> [DisableButtons]：プレーヤー操作関係のボタンをDisableにする
        /// [btnGetAlbum_Click],[InitializeForm],[ShowGrid]から呼出し
        /// </summary>
        private void DisableButtons()
        {
            btnNext.Enabled = false;
            btnPause.Enabled = false;
            btnPrevious.Enabled = false;
            btnStart.Enabled = false;
            //btnStop.Enabled = false;
        }

        /// <summary> [FillCbxCustom]メソッド
        /// [mnuCustom_Click],[InitializeForm]から呼出し
        /// </summary>
        private void FillCbxCustom()
        {
            cbxCustom.Items.Clear();
            if (!Directory.Exists(CC.CUST_FOLD)) return;

            foreach (string tfile in Directory.GetFiles(CC.CUST_FOLD))
            {
                if (Path.GetExtension(tfile).ToLower() == CC.EXT_TXT)
                    cbxCustom.Items.Add(Path.GetFileNameWithoutExtension(tfile));
            }
        }

        /// <summary> [GetAlbum]メソッド
        /// [btnGetAlbum_Click]から呼出し
        /// </summary>
        /// <param name="vfolderFP"></param>
        private void GetAlbum(string vfolderFP)
        {
            grdView.Rows.Clear();
            wmPlayer.Ctlcontrols.stop();
            pgbar.Value = 0;
            DisableButtons();
            _ratio = 1;

            var lstSongs = new List<string>();
            if (IsExistSong(vfolderFP, ref lstSongs))
            {
                _eventGrdSelection = false;
                PutData2GrdView(lstSongs);
                _eventGrdSelection = true;
                grdView.Rows[0].Selected = true;

                if (grdView.Rows.Count > 1)
                {
                    btnNext.Enabled = true;
                    btnPrevious.Enabled = true;
                }
                btnPause.Enabled = false;                
                btnStart.Enabled = true;

                txtAlbum.Text = vfolderFP.Substring(vfolderFP.LastIndexOf(@"\") + 1);
                Properties.Settings.Default.AlbumFolder = vfolderFP;
                Properties.Settings.Default.Save();

                trkbar.Value = 0;
            }
            else
            {
                txtAlbum.Text = String.Empty;
                lblArtist.Text = String.Empty;
                lblCurrent.Text = String.Empty;
                grdView.DataSource = null;
            }
        }

        /// <summary> [GetDetail]メソッド
        /// [PutData2GrdView]から呼出し
        /// </summary>
        /// <param name="vfp"></param>
        private void GetDetail(string vfp)
        {
            string folder = Path.GetDirectoryName(vfp);
            string file = Path.GetFileName(vfp);

            grdView.Rows.Add();
            DataGridViewRow grow = grdView.Rows[grdView.Rows.Count - 1];
            grow.Cells[CC.CLM_FILE].Value = vfp;
            grow.Cells[CC.CLM_VOLUME].Value = Properties.Settings.Default.StandardVolume;

            var shell = new ShellClass();
            Folder f = shell.NameSpace(folder);
            FolderItem item = f.ParseName(file);

            int count = 0;
            for (int i = 0; i < 300; i++)
            {
                string tname = f.GetDetailsOf(null, i);
                if (tname == CC.CLM_TITLE)
                {
                    grow.Cells[tname].Value = f.GetDetailsOf(item, i);
                    count += 1;
                }
                else if (tname == CC.ClmArtist)
                {
                    grow.Cells[CC.CLM_ART_XP].Value = f.GetDetailsOf(item, i);
                    count += 1;
                }
                else if (tname == CC.CLM_LENGTH)
                {
                    grow.Cells[tname].Value = f.GetDetailsOf(item, i);
                    count += 1;
                }
                if (count >= 3) break;
            }
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            SetIsWin7();
            CC.ClmArtist = (CC.IsWin7) ? CC.CLM_ART_W7 : CC.CLM_ART_XP;
            if (!CC.IsWin7) mnuStandardFolder.Enabled = false;

            txtAlbum.ReadOnly = true;
            txtAlbum.TabStop = false;
            rdoAlbum.Checked = true;
            FillCbxCustom();
            rdoCustom.Enabled = (cbxCustom.Items.Count > 0);

            DisableButtons();
            timer1.Enabled = false;
            //_sidx = -1;

            trkbar.Value = 0;

            SetGrdView();

            wmPlayer.settings.autoStart = false;
            wmPlayer.settings.playCount = PCOUNT;
        }

        /// <summary> [PlayStart]メソッド
        /// [btnNext_Click],[btnPrevious_Click],[btnStart_Click],[grdView_CellClick],
        /// [ToNext]から呼出し
        /// </summary>
        private void PlayStart()
        {
            DataGridViewRow grow = grdView.Rows[grdView.SelectedRows[0].Index];
            string filefp = grow.Cells[CC.CLM_FILE].Value.ToString();
            string artist = grow.Cells[CC.CLM_ART_XP].Value.ToString();
            string title = grow.Cells[CC.CLM_TITLE].Value.ToString();
            if (!File.Exists(filefp))
            {
                timer1.Enabled = false;
                lblArtist.Text = String.Empty;
                lblCurrent.Text = String.Empty;
                string msg = "「" + title + "」用の次のファイルが見つかりません。" 
                    + CC.NL  + CC.NL + "「" + filefp + "」";
                CC.ShowMsg(msg, CC.EXC);
                return;
            }

            lblArtist.Text = artist;
            lblCurrent.Text = title;
            Application.DoEvents();

            wmPlayer.URL = filefp;

            _volume = Convert.ToInt32(grow.Cells[CC.CLM_VOLUME].Value);
            int vol = (int)(Math.Round(_volume * _ratio));

            vol = Math.Min(vol, 100);        // Ver1.1.0で追加
            wmPlayer.settings.volume = vol;
            trkbar.Value = vol;

            int position = pgbar.Value;
            wmPlayer.Ctlcontrols.play();
            wmPlayer.Ctlcontrols.currentPosition = position;
            btnStart.Enabled = false;
            btnPause.Enabled = true;
            timer1.Enabled = true;
        }

        /// <summary> [PutData2GrdView]メソッド
        /// [GetAlbum]から呼出し
        /// </summary>
        private void PutData2GrdView(List<string> vlstSongs)
        {
            for (int i = 0; i < vlstSongs.Count; i++)
            {
                GetDetail(vlstSongs[i]);
            }
        }

        /// <summary> [ReadList]メソッド
        /// [cbxCustom_SIChanged]から呼出し
        /// </summary>
        /// <param name="vfp"></param>
        private void ReadList(string vfp, out bool vresult)
        {
            vresult = false;
            grdView.Rows.Clear();

            int stdVol = Properties.Settings.Default.StandardVolume;
            int count = 0;
            using (var reader = new StreamReader(vfp, Encoding.Default))
            {
                try
                {
                    _lstVRatio.Clear();
                    while (true)
                    {
                        string sline = reader.ReadLine();
                        if (sline == null) break;

                        string[] separator = { CC.DLMTR };
                        string[] data = sline.Split(separator, StringSplitOptions.None);

                        grdView.Rows.Add();
                        int i = grdView.Rows.Count - 1;
                        grdView[0, i].Value = data[0];
                        grdView[1, i].Value = data[1];
                        grdView[2, i].Value = data[2];
                        //grdView[3, i].Value =
                        //    (int)(_ratio * stdVol * Convert.ToDouble(data[3]));
                        double vratio = Convert.ToDouble(data[3]);
                        grdView[3, i].Value = (int)(stdVol * vratio);
                        _lstVRatio.Add(vratio);
                        grdView[4, i].Value = data[4];

                        count += 1;

                        // 以下、バグによる無限ループ回避用
                        if (count > 500)
                        {
                            CC.ShowMsg("count > 500", CC.EXC);
                            break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    CC.ShowMsg(ex.Message, CC.EXC);
                    grdView.Rows.Clear();
                    return;
                }
                finally
                {
                    reader.Close();
                }
            }
            if (count > 0)
                vresult = true;
            else
                CC.ShowMsg("このカスタムリストには有効なタイトルはありません。", CC.EXC);
        }

        /// <summary> [ResetVolume]メソッド
        /// [mnuVolume_Click]から呼出し
        /// </summary>
        private void ResetVolume()
        {
            int volume = Properties.Settings.Default.StandardVolume;
            if (rdoAlbum.Checked)
            {
                for (int i1 = 0; i1 < grdView.Rows.Count; i1++)
                {
                    grdView.Rows[i1].Cells[CC.CLM_VOLUME].Value = volume;
                }
            }
            else if (rdoCustom.Checked)
            {
                for (int i2 = 0; i2 < grdView.Rows.Count; i2++)
                {
                    grdView.Rows[i2].Cells[CC.CLM_VOLUME].Value = volume * _lstVRatio[i2];
                }
            }
        }

        /// <summary> [SetGrdView]：grdViewの初期状態を設定する
        /// [InitializeForm]から呼出し
        /// </summary>
        private void SetGrdView()
        {
            grdView.Columns.Add(CC.CLM_TITLE, CC.CLM_TITLE);
            grdView.Columns.Add(CC.CLM_ART_XP, CC.CLM_ART_XP);
            grdView.Columns.Add(CC.CLM_LENGTH, CC.CLM_LENGTH);
            grdView.Columns.Add(CC.CLM_VOLUME, CC.CLM_VOLUME);
            grdView.Columns.Add(CC.CLM_FILE, CC.CLM_FILE);

            grdView.Columns[CC.CLM_TITLE].Width = 270;
            grdView.Columns[CC.CLM_ART_XP].Width = 120;
            grdView.Columns[CC.CLM_LENGTH].Width = 85;
            grdView.Columns[CC.CLM_VOLUME].Width = 50;
            grdView.Columns[CC.CLM_FILE].Visible = false;

            CC.SetGridProperty(grdView);
            grdView.AllowUserToResizeColumns = true;
        }

        /// <summary> [SetIsWin7]メソッド
        /// [InitializeForm]から呼出し
        /// </summary>
        private void SetIsWin7()
        {
            var os = System.Environment.OSVersion;
            System.PlatformID pid = os.Platform;
            int major = os.Version.Major;
            int minor = os.Version.Minor;

            CC.IsWin7 = (pid == System.PlatformID.Win32NT && major == 6); // Win8対応
        }

        /// <summary> [ToNext]メソッド
        /// [btnNext_Click],[timer1_Tick]から呼出し
        /// </summary>
        private void ToNext()
        {
            pgbar.Value = 0;

            int count = grdView.Rows.Count;
            if (count == 0) return;
            if (count == 1)
            {
                wmPlayer.Ctlcontrols.stop();
                PlayStart();
                return;
            }

            int idx = grdView.SelectedRows[0].Index;
            int sidx = 0;
            if (idx == count - 1)
            {
                grdView.Rows[0].Selected = true;
            }
            else
            {
                grdView.Rows[idx + 1].Selected = true;
                sidx = idx + 1;
            }
            grdView.FirstDisplayedScrollingRowIndex = sidx;
            PlayStart();
        }

        /// <summary> [ToPrevious]メソッド
        /// [btnPrevious_Click]から呼出し
        /// </summary>
        private void ToPrevious()
        {
            pgbar.Value = 0;

            int count = grdView.Rows.Count;
            if (count <= 1) return;
            
            int idx = grdView.SelectedRows[0].Index;
            int sidx = count - 1;
            if (idx == 0)
            {
                grdView.Rows[count - 1].Selected = true;
                //_sidx = count - 1;
            }
            else
            {
                grdView.Rows[idx - 1].Selected = true;
                //_sidx = idx - 1;
                sidx = idx - 1;
            }
            //grdView.FirstDisplayedScrollingRowIndex = _sidx;
            grdView.FirstDisplayedScrollingRowIndex = sidx;
            PlayStart();
        }

    }
}
