﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Shell32;
using System.IO;

namespace CKMusicPlayer
{
    public partial class F10_CustomList : Form
    {
        /// <summary> [コンストラクター] </summary>
        public F10_CustomList()
        {
            InitializeComponent();
        }

        /// <summary> [CreatedOrDeleted]プロパティ(読取り専用) </summary>
        public bool CreatedOrDeleted
        {
            get { return _createdOrDeleted; }
        }

        /// <summary> [CustomList]プロパティ(書込み専用) </summary>
        public string CustomList
        {
            set { _customlist = value; }
        }

        enum UpDown { Up, Down };

        const string CK_OF = "□";
        const string CK_ON = "■";
        const string CLM_CHK = "Check";
        const string MD_CRT = "作成";
        const string MD_UPD = "更新";

        private bool _createdOrDeleted = false;     // [CreatedOrDeleted]プロパティ用
        private bool _enableCbxEvent = true;
        private bool _updatePlayList = false;
        private string _customlist = String.Empty;  // [CustomList]プロパティ用

        /// <summary> [追加]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAddFile_Click(object sender, EventArgs e)
        {
            using (var sform = new F14_AddFile())
            {
                if (sform.ShowDialog() == CC.RSLT_OK)
                {
                    List<CMusicFile> lst = sform.ListMF;
                    AddFile2Grid(sform.ListMF);
                }
            }
        }

        /// <summary> [戻る]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBack_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary> [クリア]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClearFile_Click(object sender, EventArgs e)
        {
            // クリア確認
            if (CC.MsgCanceled("プレイリストの表示を消去します。")) return;

            // クリア実行
            grdView.Rows.Clear();
        }

        /// <summary> [作成]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCreate_Click(object sender, EventArgs e)
        {
            if (grdView.Rows.Count == 0)   // 通常、この事態は発生しない
            {
                CC.ShowMsg("プレイリストがありません。", CC.EXC);
                return;
            }
            
            string newName = txtNew.Text.Trim();
            if (newName.Length == 0)
            {
                CC.ShowAlarm(txtNew, "リスト名を入力してください。");
                txtNew.Focus();
                return;
            }

            // 各曲のファイルの存在チェック
            if (!CheckFile(MD_CRT))
            {
                CC.ShowMsg("リストの作成を中断しました。", CC.EXC);
                return;
            }

            // 作成確認
            string msg = "[" + newName + "] のリストを作成します。";
            if (CC.MsgCanceled(msg)) return;            

            bool result;
            CreateList(out result);
            if (result)
            { 
                FillCbxCustom();
                if (!rdoEdit.Enabled) rdoEdit.Enabled = true;
                _createdOrDeleted = true;
                _updatePlayList = false;
            }
        }

        /// <summary> [(カスタムリスト)削除]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDeleteList_Click(object sender, EventArgs e)
        {
            if (cbxCustom.SelectedIndex < 0) return;    // 通常、この事態は発生しない

            string fname = cbxCustom.Text;
            string filefp = CC.CUST_FOLD + @"\" + fname + ".txt";
            if (!File.Exists(filefp))                   // 通常、この事態も発生しない
            {
                cbxCustom.BackColor = Color.Red;
                CC.ShowMsg("このリストは存在しません。", CC.EXC);
                cbxCustom.BackColor = CC.COL_WIN;
                cbxCustom.SelectedIndex = -1;
                return;
            }

            // 削除確認
            string msg = "カスタムリスト [" + fname + "] を削除します。";
            if (CC.MsgCanceled(msg)) return;

            // 削除実行
            if (grdView.Rows.Count > 0) grdView.Rows.Clear();
            File.Delete(filefp);
            FillCbxCustom();
            _createdOrDeleted = true;
        }

        /// <summary> [↓]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDown_Click(object sender, EventArgs e)
        {
            int idx = grdView.SelectedRows[0].Index;
            if (idx == -1 || idx == grdView.Rows.Count - 1) 
                return;      // 通常、この事態は発生しない

            ChangeGridData(idx);

            grdView.Rows[idx + 1].Selected = true;
            GridScroll(UpDown.Down, idx + 1);
            EnableBtnUpDown(idx + 1);
        }

        /// <summary> [編集]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEditFile_Click(object sender, EventArgs e)
        {
            using (var sform = new F12_EditCustom())
            {
                DataGridViewRow grow = grdView.Rows[grdView.SelectedRows[0].Index];
                sform.Artist = grow.Cells[CC.CLM_ART_XP].Value.ToString();
                sform.Title = grow.Cells[CC.CLM_TITLE].Value.ToString();
                sform.Ratio = Convert.ToDecimal(grow.Cells[CC.CLM_VRATIO].Value);
                string originalfp = grow.Cells[CC.CLM_FILE].Value.ToString();
                sform.SFile = originalfp;
                sform.FLeft = Math.Max(0, this.Left - 30);
                sform.FTop = this.Top + pnlMain.Top + gbxList.Top + 30;

                this.Enabled = false;
                if (sform.ShowDialog() == CC.RSLT_OK)
                {
                    grow.Cells[CC.CLM_ART_XP].Value = sform.Artist;
                    grow.Cells[CC.CLM_TITLE].Value = sform.Title;
                    grow.Cells[CC.CLM_VRATIO].Value = sform.Ratio;
                    string newfp = sform.SFile.ToLower();                    
                    grow.Cells[CC.CLM_FILE].Value = newfp;
                    _updatePlayList = true;
                }
                this.Enabled = true;
            }
            grdView.Focus();
        }

        /// <summary> [Remove]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoveFile_Click(object sender, EventArgs e)
        {
            int sidx = grdView.SelectedRows[0].Index;
            DataGridViewRow grow = grdView.Rows[sidx];
            // 削除確認
            string title = grow.Cells[CC.CLM_TITLE].Value.ToString();
            string msg = "「" + title + "」" + CC.NL + "を削除します。";
            if (CC.MsgCanceled(msg)) return;  

            // 削除実行
            grdView.Rows.Remove(grow);
            int count = grdView.Rows.Count;
            if (count > 0)
            {
                if (sidx == count) sidx = count - 1;
                grdView.Rows[sidx].Selected = true;
            }
        }

        /// <summary> [↑]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUp_Click(object sender, EventArgs e)
        {
            int idx = grdView.SelectedRows[0].Index;
            if (idx < 1) return;  // 通常、この事態は発生しない

            ChangeGridData(idx - 1);
            
            grdView.Rows[idx - 1].Selected = true;
            GridScroll(UpDown.Up, idx - 1);
            EnableBtnUpDown(idx - 1);
        }

        /// <summary> [更新]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpdate_Click(object sender, EventArgs e)
        {
            if (grdView.Rows.Count == 0)      // 通常、この事態は発生しない
            {
                CC.ShowMsg("プレイリストがありません。", CC.EXC);
                return;
            }
            if (cbxCustom.SelectedIndex < 0)  // 通常、この事態も発生しない
            {
                CC.ShowMsg("リスト名を選択してください。", CC.EXC);
                cbxCustom.Focus();
                return;
            }

            UpdateList();
        }

        /// <summary> [cbxCustom_SIChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxCustom_SIChanged(object sender, EventArgs e)
        {
            if (cbxCustom.SelectedIndex < 0) return;
            if (!_enableCbxEvent) return;

            string customlist = cbxCustom.Text;
            // メイン画面で呼び出されているカスタムリストは更新・削除しない
            if (customlist.ToLower() == _customlist.ToLower())
            {
                string msg = "このリストはメイン画面で呼び出されているので編集できません。";
                CC.ShowMsg(msg, CC.EXC);
                //cbxCustom.SelectedIndex = -1;
                UndoCbxCustom();
                return;
            }

            string filefp = CC.CUST_FOLD + @"\" + cbxCustom.Text + ".txt";
            if (!File.Exists(filefp))      // 通常、この事態は発生しない
            {
                cbxCustom.BackColor = Color.Red;
                CC.ShowMsg("このリストは存在しません。", CC.EXC);
                cbxCustom.BackColor = CC.COL_WIN;
                //cbxCustom.SelectedIndex = -1;
                UndoCbxCustom();
                return;
            }

            // grdViewクリアの確認
            if (grdView.Rows.Count > 0)
            {
                string msg = "カスタムリストを呼び出すと表示されているプレイリストは"
                    + "クリアされます。" + CC.NL + "リスト呼び出しを実行しますか？";
                if (CC.MsgCanceled(msg))
                {
                    UndoCbxCustom();
                    return; 
                }
                grdView.Rows.Clear();
            }

            // カスタムリストの読み出し
            bool result;
            ReadList(filefp, out result);
            _updatePlayList = result && (grdView.Rows.Count > 0);
            btnDeleteList.Enabled = true;
            cbxCustom.Tag = cbxCustom.Text;
        }

        /// <summary> [Form_Closing]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closing(object sender, FormClosingEventArgs e)
        {
            if (_updatePlayList && grdView.Rows.Count > 0)
            {
                string msg = "プレイリストの編集結果が保存されていませんが、"
                    + CC.NL + "このまま画面を閉じますか？";
                if (CC.MsgCanceled(msg, true))
                    e.Cancel = true;
            }
        }

        /// <summary> [Form_Load]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            //CC.SetForm(this);
            this.StartPosition = FormStartPosition.CenterParent;
            InitializeForm();
        }

        /// <summary> [grdView_CellClick]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                if (grdView.SelectedRows.Count > 0)
                {
                    EnableBtnRemoveEditFile(true);
                    EnableBtnUpDown(grdView.SelectedRows[0].Index);
                }
                else
                {
                    EnableBtnUpDown(-1);
                }
            }
            catch (Exception ex)
            {
                CC.ShowMsg(ex.Message, CC.EXC);
            }
        }

        /// <summary> [grdView_KeyDown]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_KeyDown(object sender, KeyEventArgs e)
        {
            CC.CtrlKeyDown(this, e);
        }

        /// <summary> [grdView_RowsAdded]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            btnClearFile.Enabled = true;
            _updatePlayList = true;
            if (rdoNew.Checked || cbxCustom.SelectedIndex >= 0)
                EnableBtnCreate(rdoNew.Checked);            
        }

        /// <summary> [grdView_RowsRemoved]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
        {
            if (grdView.Rows.Count == 0)
            {
                btnCreate.Enabled = false;
                btnDeleteList.Enabled = (cbxCustom.SelectedIndex >= 0);
                btnUpdate.Enabled = false;
                btnClearFile.Enabled = false;
                EnableBtnRemoveEditFile(false);
                EnableBtnUpDown(-1);
            }
            else
            {
                _updatePlayList = true;
            }
        }

        /// <summary> [Rdo_CheckedChanged]イベント
        /// [rdoEdit],[rdoNew]に対して適用
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Rdo_CheckedChanged(object sender, EventArgs e)
        {
            var rbtn = (RadioButton)sender;
            if (!rbtn.Checked) return;

            btnDeleteList.Enabled = false;
            btnUpdate.Enabled = false;
            
            if (rbtn == rdoNew)
            {
                btnCreate.Enabled = (grdView.Rows.Count > 0);
                txtNew.Enabled = true;
                cbxCustom.SelectedIndex = -1;
                cbxCustom.Tag = String.Empty;
                cbxCustom.Enabled = false;
                lblList2.Enabled = false;
                lblList1.Enabled = true;
                txtNew.Focus();
            }
            else
            {
                btnCreate.Enabled = false;
                txtNew.Text = String.Empty;
                txtNew.Enabled = false; ;
                cbxCustom.Enabled = true;
                lblList2.Enabled = true;
                lblList1.Enabled = false;
            }
        }

        /// <summary> [txtNew_Enter]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtNew_Enter(object sender, EventArgs e)
        {
            txtNew.SelectAll();
        }

        /// <summary> [txtNew_KeyDown]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtNew_KeyDown(object sender, KeyEventArgs e)
        {
            CC.CtrlKeyDown(this, e);
        }

        /// <summary> [txtNew_Validating]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtNew_Validating(object sender, CancelEventArgs e)
        {
            if (this.ActiveControl == txtNew) return;  // 閉じるボタンで閉じるとき
            if (this.ActiveControl == btnBack) return;

            // ファイル名として不適切な文字が使われている場合の処置
            string lname = txtNew.Text.Trim();
            if (lname.Length > 0)
            {
                if (!CC.CheckNoInvalidFileNameChars(txtNew, lname))
                    e.Cancel = true;                
            }
        }


        /// <summary> [CheckFile]メソッド
        /// [btnUpdate_Click]から呼出し
        /// </summary>
        /// <returns></returns>
        private bool CheckFile(string vmd)
        {
            bool result = true;
            string mfile = String.Empty;
            string title = String.Empty;
            string msg = String.Empty;
            for (int i = 0; i < grdView.Rows.Count; i++)
            {
                mfile = grdView.Rows[i].Cells[CC.CLM_FILE].Value.ToString();
                if (!File.Exists(mfile))
                {
                    title = grdView.Rows[i].Cells[CC.CLM_TITLE].Value.ToString();
                    msg = "「" + title + "」用の次のファイルが見つかりません。"
                        + CC.NL + CC.NL + "「" + mfile + "」"
                        + CC.NL + CC.NL + "カスタムリストの" + vmd + "を続けますか？";
                    if (CC.MsgCanceled(msg, true))
                    {
                        result = false;
                        break;
                    }
                }
            }
            return result;
        }

        /// <summary> [IsExistSong]メソッド
        /// [btnGetAlbum_Click]から呼出し
        /// </summary>
        /// <param name="vfp">フォルダー(フルパス)</param>
        /// <param name="vlstSongs">音楽ファイルを格納するリスト</param>
        /// <returns>true:フォルダー中に音楽ファイルが存在する場合</returns>
        private bool IsExistSong(string vfp, ref List<string> vlstSongs)
        {
            if (vfp.Length == 0) return false;

            vlstSongs = new List<string>();
            foreach (string sfile in Directory.GetFiles(vfp))
            {
                for (int i = 0; i < CC.EXTS.Length; i++)
                {
                    string ext = Path.GetExtension(sfile).ToLower();
                    if (Path.GetExtension(sfile).ToLower() == CC.EXTS[i])
                    {
                        vlstSongs.Add(sfile.ToLower()); // ファイル名は小文字で取り込む
                        break;
                    }
                }
            }
            return (vlstSongs.Count > 0);
        }

        /// <summary> [AddFile2Grid]メソッド
        /// [btnAddFile_Click]から呼出し
        /// </summary>
        /// <param name="vlst"></param>
        private void AddFile2Grid(List<CMusicFile> vlst)
        {
            for (int i = 0; i < vlst.Count; i++)
            {
                grdView.Rows.Add();
                DataGridViewRow grow = grdView.Rows[grdView.Rows.Count - 1];
                grow.Cells[CC.CLM_TITLE].Value = vlst[i].Title;
                grow.Cells[CC.CLM_ART_XP].Value = vlst[i].Artist;
                grow.Cells[CC.CLM_LENGTH].Value = vlst[i].Length;
                grow.Cells[CC.CLM_VRATIO].Value = vlst[i].VRatio;
                grow.Cells[CC.CLM_FILE].Value = vlst[i].File;
            }
            grdView.ClearSelection();
        }

        /// <summary> [ChangeGridData]：grdViewの行データを上下入れ替える
        /// [btnDown_Click],[btnUp_Click]から呼出し
        /// </summary>
        /// <param name="vidx"></param>
        private void ChangeGridData(int vidx)
        {
            DataGridViewRow gr1 = grdView.Rows[vidx];
            DataGridViewRow gr2 = grdView.Rows[vidx + 1];
            string title = gr2.Cells[CC.CLM_TITLE].Value.ToString();
            string artist = gr2.Cells[CC.CLM_ART_XP].Value.ToString();
            string length = gr2.Cells[CC.CLM_LENGTH].Value.ToString();
            decimal ratio = Convert.ToDecimal(gr2.Cells[CC.CLM_VRATIO].Value);
            string filefp = gr2.Cells[CC.CLM_FILE].Value.ToString();
            gr2.Cells[CC.CLM_TITLE].Value = gr1.Cells[CC.CLM_TITLE].Value.ToString();
            gr2.Cells[CC.CLM_ART_XP].Value = gr1.Cells[CC.CLM_ART_XP].Value.ToString();
            gr2.Cells[CC.CLM_LENGTH].Value = gr1.Cells[CC.CLM_LENGTH].Value.ToString();
            gr2.Cells[CC.CLM_VRATIO].Value = Convert.ToDecimal(gr1.Cells[CC.CLM_VRATIO].Value);
            gr2.Cells[CC.CLM_FILE].Value = gr1.Cells[CC.CLM_FILE].Value.ToString();
            gr1.Cells[CC.CLM_TITLE].Value = title;
            gr1.Cells[CC.CLM_ART_XP].Value = artist;
            gr1.Cells[CC.CLM_LENGTH].Value = length;
            gr1.Cells[CC.CLM_VRATIO].Value = ratio;
            gr1.Cells[CC.CLM_FILE].Value = filefp;
        }

        /// <summary> [CreateList]メソッド
        /// [btnCreate_Click]から呼出し
        /// </summary>
        /// <param name="vresult"></param>
        private void CreateList(out bool vresult)
        {
            vresult = false;

            // [Application.StartupPath + "\Custom"]がない場合には作成する
            if (!Directory.Exists(CC.CUST_FOLD))
                Directory.CreateDirectory(CC.CUST_FOLD);

            string fname = txtNew.Text.Trim();
            string filefp =CC.CUST_FOLD + @"\" + fname + ".txt";

            if (File.Exists(filefp))
            {
                // ファイル上書きの確認
                string msg = "[" + fname + "] のリストは作成済みです。"
                    + CC.NL + "リストを上書き更新しますか？";
                if (CC.MsgCanceled(msg))
                {
                    txtNew.Focus();
                    return; 
                }
            }

            // カスタムリストファイル作成
            try
            {
                using (var writer = new StreamWriter(filefp, false, Encoding.Default))
                {
                    for (int i1 = 0; i1 < grdView.Rows.Count; i1++)
                    {
                        string sline = grdView[0, i1].Value.ToString();
                        for (int i2 = 1; i2 < grdView.Columns.Count; i2++)
                        {
                            sline += CC.DLMTR + grdView[i2, i1].Value.ToString();
                        }
                        writer.WriteLine(sline);
                    }
                    writer.Close();
                }
                CC.ShowMsg("リスト作成完了。", CC.INF);
                vresult = true;
            }
            catch (Exception ex)
            {
                CC.ShowMsg(ex.Message, CC.EXC);
                vresult = false;
            }
        }

        /// <summary> [EnableBtnCreate]：btnCreate,btnDeleteFile,btnUpdateのEnabledを設定する
        /// [grdView_RowsAdded]から呼出し
        /// </summary>
        /// <param name="vbool"></param>
        private void EnableBtnCreate(bool vbool)
        {
            btnCreate.Enabled = vbool;
            btnDeleteList.Enabled = !vbool;
            btnUpdate.Enabled = !vbool;
        }

        /// <summary> [EnableBtnRemoveEditFile]メソッド
        /// [grdView_CellClick],[grdView_RowsRemoved]から呼出し
        /// </summary>
        /// <param name="vbool"></param>
        private void EnableBtnRemoveEditFile(bool vbool)
        {
            btnRemoveFile.Enabled = vbool;
            btnEditFile.Enabled = vbool;
            //EnableBtnUpDown(grdList.SelectedRows[0].Index);
        }

        /// <summary> [EnableBtnUpDown]：btnDown,btnUpのEnabledを設定するメソッド
        /// [btnDown_Click],[btnUp_Click],[grdView_CellClick],[grdView_RowsRemoved]から呼出し
        /// </summary>
        /// <param name="vidx"></param>
        private void EnableBtnUpDown(int vidx)
        {
            if (vidx == 0)
            {
                btnUp.Enabled = false;
                btnDown.Enabled = true;
            }
            else if (vidx == -1)
            {
                btnUp.Enabled = false;
                btnDown.Enabled = false;
            }
            else if (vidx == grdView.Rows.Count - 1)
            {
                btnUp.Enabled = true;
                btnDown.Enabled = false;
            }
            else
            {
                btnUp.Enabled = true;
                btnDown.Enabled = true;
            }
        }

        /// <summary> [FillCbxCustom]メソッド
        /// [btnCreate_Click],[btnDeleteList_Click],[InitializeForm]から呼出し
        /// </summary>
        private void FillCbxCustom()
        {
            cbxCustom.Items.Clear();
            btnDeleteList.Enabled = false;
            if (!Directory.Exists(CC.CUST_FOLD)) return;

            foreach (string tfile in Directory.GetFiles(CC.CUST_FOLD))
            {
                if (Path.GetExtension(tfile).ToLower() == CC.EXT_TXT)
                    cbxCustom.Items.Add(Path.GetFileNameWithoutExtension(tfile));
            }
        }

        private void GridScroll(UpDown vud, int vidx)
        {
            const int GRID_ROW_COUNT = 16;

            int fsidx = grdView.FirstDisplayedScrollingRowIndex;
            if (vud == UpDown.Up)         // btnUpからの呼び出しの場合
            {
                if (vidx < fsidx)
                    grdView.FirstDisplayedScrollingRowIndex = vidx;
            }
            else if (vud == UpDown.Down)  // btnUpからの呼び出しの場合
            {
                if (vidx > fsidx + GRID_ROW_COUNT - 1)
                    grdView.FirstDisplayedScrollingRowIndex = vidx - GRID_ROW_COUNT + 1;
            }
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            rdoNew.Checked = true;
            FillCbxCustom();
            rdoEdit.Enabled = (cbxCustom.Items.Count > 0);
            btnClearFile.Enabled = false;
            btnDeleteList.Enabled = false;
            btnRemoveFile.Enabled = false;
            btnEditFile.Enabled = false;
            btnUp.Enabled = false;
            btnDown.Enabled = false;

            SetGrdView();
        }

        /// <summary> [ReadList]メソッド
        /// [cbxCustom_SIChanged]から呼出し
        /// </summary>
        /// <param name="vfp"></param>
        /// <param name="vresult"></param>
        private void ReadList(string vfp, out bool vresult)
        {
            using (var reader = new StreamReader(vfp, Encoding.Default))
            {
                int count = 0;
                vresult = false;
                try
                {
                    while (true)
                    {
                        string sline = reader.ReadLine();
                        if (sline == null) break;

                        string[] separator = {CC.DLMTR};
                        string[] data = sline.Split(separator, StringSplitOptions.None);

                        grdView.Rows.Add();
                        count += 1;
                        int i = grdView.Rows.Count - 1;
                        grdView[0, i].Value = data[0];
                        grdView[1, i].Value = data[1];
                        grdView[2, i].Value = data[2];
                        grdView[3, i].Value = Convert.ToDecimal(data[3]);
                        grdView[4, i].Value = data[4];

                        // 以下、バグによる無限ループ回避用
                        if (count > 500)
                        {
                            CC.ShowMsg("count > 500", CC.EXC);
                            break;
                        }
                    }
                    if (count > 0)
                        vresult = true;
                    else
                        CC.ShowMsg("このカスタムリストには有効なタイトルはありません。", CC.EXC);
                }
                catch (Exception ex)
                {
                    CC.ShowMsg(ex.Message, CC.EXC);
                    // このメソッドで追加した行を削除する
                    for (int i = 0; i < count; i++)
                    {
                        grdView.Rows.RemoveAt(grdView.Rows.Count - 1);
                    }
                    //return;
                }
                finally
                {
                    reader.Close();
                } 
            }
            grdView.ClearSelection();
        }

        /// <summary> [SetGrdView]：grdViewの初期状態を設定する
        /// [InitializeForm]から呼出し
        /// </summary>
        private void SetGrdView()
        {
            grdView.Columns.Add(CC.CLM_TITLE, CC.CLM_TITLE);
            grdView.Columns.Add(CC.CLM_ART_XP, CC.CLM_ART_XP);
            grdView.Columns.Add(CC.CLM_LENGTH, CC.CLM_LENGTH);
            grdView.Columns.Add(CC.CLM_VRATIO, CC.CLM_VRATIO);
            grdView.Columns.Add(CC.CLM_FILE, CC.CLM_FILE);

            grdView.Columns[CC.CLM_TITLE].Width = 306;
            grdView.Columns[CC.CLM_ART_XP].Width = 120;
            grdView.Columns[CC.CLM_VRATIO].Width = 75;
            grdView.Columns[CC.CLM_VRATIO].DefaultCellStyle.Format = "#0.0";
            grdView.Columns[CC.CLM_LENGTH].Visible = false;
            grdView.Columns[CC.CLM_FILE].Visible = false;

            CC.SetGridProperty(grdView);
            grdView.AllowUserToResizeColumns = true;
        }

        /// <summary> [UndoCbxCustom]：cbxCustomを変更前の状態に戻すメソッド
        /// [cbxCustom_SIChanged]
        /// </summary>
        private void UndoCbxCustom()
        {
            string previous = cbxCustom.Tag.ToString();
            if (previous.Length == 0)
                cbxCustom.SelectedIndex = -1;
            else
            {
                _enableCbxEvent = false;
                cbxCustom.Text = previous;
                _enableCbxEvent = true;
            }
        }

        /// <summary> [UpdateList]メソッド
        /// [btnUpdate_Click]から呼出し
        /// </summary>
        private void UpdateList()
        {
            if (!Directory.Exists(CC.CUST_FOLD))   // 通常、この事態は発生しない
                Directory.CreateDirectory(CC.CUST_FOLD);

            string fname = cbxCustom.Text;
            string filefp = CC.CUST_FOLD + @"\" + fname + ".txt";

            // 各曲のファイルの存在チェック
            if (!CheckFile(MD_UPD))
            { 
                CC.ShowMsg("リストの更新を中断しました。", CC.EXC);
                return; 
            }
            
            // 更新確認
            string msg = "[" + fname + "] のリストを更新します。";
            if (CC.MsgCanceled(msg))  return;            

            // 更新実行
            bool result = true;
            try
            {
                using (var writer = new StreamWriter(filefp, false, Encoding.Default))
                {
                    for (int i1 = 0; i1 < grdView.Rows.Count; i1++)
                    {
                        string sline = grdView[0, i1].Value.ToString();
                        for (int i2 = 1; i2 < grdView.Columns.Count; i2++)
                        {
                            sline += CC.DLMTR + grdView[i2, i1].Value.ToString();
                        }
                        writer.WriteLine(sline);
                    }
                    writer.Close();
                }
                if (result)
                {
                    CC.ShowMsg("リスト更新完了。", CC.INF);
                    _updatePlayList = false;
                }
            }
            catch (Exception ex)
            {
                CC.ShowMsg(ex.Message, CC.EXC);
            }
        }

    }
}
