﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;

namespace CKMusicPlayer
{
    public partial class F12_EditCustom : Form
    {
        /// <summary> [コンストラクター] </summary>
        public F12_EditCustom()
        {
            InitializeComponent();
        }

        /// <summary> [Ratio]プロパティ </summary>
        public decimal Ratio
        {
            get { return _ratio; }
            set { _ratio = value; }
        }

        /// <summary> [FLeft]プロパティ(書込み専用) </summary>
        public int FLeft
        {
            set { _fleft = value; }
        }

        /// <summary> [FTop]プロパティ(書込み専用) </summary>
        public int FTop
        {
            set { _ftop = value; }
        }

        /// <summary> [Artist]プロパティ </summary>
        public string Artist
        {
            get { return _artist; }
            set { _artist = value; }
        }

        /// <summary> [SFile]プロパティ </summary>
        public string SFile
        {
            get { return _file; }
            set { _file = value; }
        }

        /// <summary> [Title]プロパティ </summary>
        public string Title
        {
            get { return _title; }
            set { _title = value; }
        }

        private const string DEF_FOLDER = "C:\\";

        private bool _initialized = false;
        private bool _updated = false;
        private decimal _ratio;
        private int _fleft;
        private int _ftop;
        private string _artist = String.Empty;
        private string _file = String.Empty;
        private string _title = String.Empty;

        /// <summary> [キャンセル]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (_updated)
            {
                if (CC.MsgCanceled("項目が編集されていますが、キャンセルしますか？"))
                    return;
            }
            this.DialogResult = CC.RSLT_CANCEL;
            this.Close();
        }

        /// <summary> [OK]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            string filefp = txtFile.Text.Trim().ToLower();
            if (filefp.Length == 0)
            {
                CC.ShowAlarm(txtFile, "ファイル名を入力してください。");
                btnGetFile.Focus();
                return; 
            }

            if (!File.Exists(filefp))
            {
                CC.ShowAlarm(txtFile, "このファイルは存在しません。");
                btnGetFile.Focus();
                return;
            }

            _artist = txtArtist.Text.Trim();
            _title = txtTitle.Text.Trim();
            _ratio = nudRatio.Value;
            _file = filefp;

            Properties.Settings.Default.F12_OpenFolder = Path.GetDirectoryName(_file);
            Properties.Settings.Default.Save();

            this.DialogResult = CC.RSLT_OK;
            this.Close();
        }

        /// <summary> [参照]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetFile_Click(object sender, EventArgs e)
        {
            if (File.Exists(_file))
            {
                openFileDlg.InitialDirectory = Path.GetDirectoryName(_file);
                openFileDlg.FileName = Path.GetFileName(_file);
                //openFileDlg.FileName = _file;
            }
            else
            {
                openFileDlg.InitialDirectory = Properties.Settings.Default.F12_OpenFolder;
                openFileDlg.FileName = String.Empty;
            }

            if (openFileDlg.ShowDialog() == CC.RSLT_OK)
            {
                string filefp = openFileDlg.FileName;
                if (!IsMusicFile(filefp))
                {
                    string msg = "このファイルは対象となる音楽ファイルではありません。";
                    CC.ShowMsg(msg, CC.EXC);
                }
                else
                {
                    txtFile.Text = filefp;
                }
            }
        }

        /// <summary> [Ctrl_KeyDown]イベント
        /// nudRatio,txtArtist,txtEnter,txtTitleに対して適用
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ctrl_KeyDown(object sender, KeyEventArgs e)
        {
            CC.CtrlKeyDown(this, e);
        }

        /// <summary> [Form_Load]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            this.Left = _fleft;
            this.Top = _ftop;
            InitializeForm();
        }

        /// <summary> [nudRatio_ValueChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void nudRatio_ValueChanged(object sender, EventArgs e)
        {
            if (_initialized) _updated = true;
        }

        /// <summary> [txtArtist_TextChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtArtist_TextChanged(object sender, EventArgs e)
        {
            if (_initialized) _updated = true;
        }

        /// <summary> [TB_Enter]イベント
        /// txtArtist,txtEnter,txtTitleに対して適用
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TB_Enter(object sender, EventArgs e)
        {
            ((TextBox)sender).SelectAll();
        }

        /// <summary> [txtFile_TextChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtFile_TextChanged(object sender, EventArgs e)
        {
            if (_initialized && txtFile.Text.Trim().Length > 0) _updated = true;
        }

        /// <summary> [txtFile_Validating]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtFile_Validating(object sender, CancelEventArgs e)
        {
            if (this.ActiveControl == txtFile) return;  // 閉じるボタンで閉じるとき
            if (this.ActiveControl == btnCancel) return;
            if (this.ActiveControl == btnGetFile) return;

            string filefp = txtFile.Text.Trim();
            if (filefp.Length == 0) return;

            if (!File.Exists(filefp))
            {
                CC.ShowAlarm(txtFile, "このファイルは存在しません。");
                e.Cancel = true;
            }
        }

        /// <summary> [txtTitle_TextChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtTitle_TextChanged(object sender, EventArgs e)
        {
            string title = txtTitle.Text.Trim();
            btnOK.Enabled = (title.Length > 0);
            if (_initialized && title.Length > 0) _updated = true;
        }


        /// <summary> [IsMusicFile]メソッド
        /// [btnGetFile_Click]から呼出し
        /// </summary>
        /// <param name="vfile"></param>
        /// <returns></returns>
        private bool IsMusicFile(string vfile)
        {
            for (int i = 0; i < CC.EXTS.Length; i++)
            {
                if (Path.GetExtension(vfile).ToLower() == CC.EXTS[i])
                    return true;
            }
            return false;
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            _initialized = false;
            txtTitle.Text = _title;
            txtArtist.Text = _artist;
            nudRatio.Value = _ratio;
            txtFile.Text = _file;
            _initialized = true;
        }

    }
}
